#include <algorithm>
#include <cassert>
#include <cstdio>
#include <functional>
#include <string>
#include <vector>

using i64 = long long;

struct Node {
  int go[2];
  i64 len[2];
  i64 ways;
  bool acc;
} nodes[300];

void solve_small(int n, int mod, const std::vector<i64> &p) {
  static std::vector<std::string> fib = {"b", "a", "ab", "aba", "abaab"};
  std::string s = fib[n];
  std::vector<std::string> suffix;
  for (int i = 0; i < (int)s.size(); ++i)
    suffix.push_back(s.substr(i));
  std::sort(suffix.begin(), suffix.end());
  for (auto &w : p) {
    printf("%d ", int(s.size() - suffix[w - 1].size() + 1) % mod);
  }
  puts("");
}

// f(0) = 0, f(1) = 1, f(n) = f(n - 1) + f(n - 2)
std::pair<i64, i64> fib(i64 n, i64 mod) {
  if (n == 0)
    return {0, 1};
  i64 x, y;
  if (n & 1) {
    std::tie(y, x) = fib(n - 1, mod);
    return {x, (y + x) % mod};
  } else {
    std::tie(x, y) = fib(n >> 1, mod);
    return {(x * y + x * (y - x + mod)) % mod, (x * x + y * y) % mod};
  }
}

std::pair<int, i64> build(i64 n, i64 mod, i64 bound) {
  int parity = n % 2;
  for (int i = 0; i < 300; ++i) {
    nodes[i].go[0] = nodes[i].go[1] = -1;
    nodes[i].len[0] = nodes[i].len[1] = 0;
    nodes[i].ways = nodes[i].acc = 0;
  }
  int m = 0;
  for (; n >= 5; --n) {
    if (n % 2 == parity) {
      nodes[m].acc = true;
    }
    if (m) {
      nodes[m].go[0] = m - 1;
      nodes[m].len[0] = fib(n - 1, mod).second - 2;
    }
    if (n & 1) { // end with ba
      if (m) {
        nodes[m + 2].go[0] = m - 2;
        nodes[m + 2].len[0] = 1;
      }
      nodes[m + 2].go[1] = m + 1;
      nodes[m + 1].go[0] = m;
      nodes[m + 2].len[1] = nodes[m + 1].len[0] = 1;
    } else { // end with ab
      if (m) {
        nodes[m + 2].go[1] = m - 2;
        nodes[m + 2].len[1] = 1;
      }
      nodes[m + 2].go[0] = m + 1;
      nodes[m + 1].go[1] = m;
      nodes[m + 2].len[0] = nodes[m + 1].len[1] = 1;
    }
    for (int i = 0; i < 3; ++i) {
      nodes[i + m].ways = nodes[i + m].acc;
      for (int e = 0; e < 2; ++e)
        if (nodes[i + m].go[e] != -1) {
          int x = nodes[i + m].go[e];
          nodes[i + m].ways += nodes[x].ways;
        }
    }
    if (nodes[m].ways >= bound && n % 2 == 0) {
      for (int i = m; i >= 0; --i) {
        // printf("%d: 0-> [%d,%lld] 1->[%d,%lld], ways=%lld acc=%d\n", i,
        // nodes[i].go[0], nodes[i].len[0], nodes[i].go[1], nodes[i].len[1],
        // nodes[i].ways, (int)nodes[i].acc);
      }
      return {m, n};
    }
    m += 3;
  }
  if (parity == 0) {
    nodes[m].acc = nodes[m + 3].acc = 1;
  } else {
    nodes[m + 2].acc = nodes[m + 4].acc = 1;
  }
  nodes[m + 5].go[1] = m + 3;
  nodes[m + 5].len[1] = 1;
  nodes[m + 2].go[1] = m - 2;
  nodes[m + 2].len[1] = 1;
  nodes[m + 4].go[0] = m + 1;
  nodes[m + 4].len[0] = 1;
  static std::string s = "abaaba";
  for (int i = 0; i <= 5; ++i) {
    int e = s[5 - i] - 'a';
    nodes[m + i].go[e] = m + i - 1;
    nodes[m + i].len[e] = 1;
    nodes[i + m].ways = nodes[i + m].acc;
    for (int e = 0; e < 2; ++e)
      if (nodes[i + m].go[e] != -1) {
        int x = nodes[i + m].go[e];
        nodes[i + m].ways += nodes[x].ways;
      }
  }
  for (int i = m + 5; i >= 0; --i) {
    // printf("%d: 0-> [%d,%lld] 1->[%d,%lld], ways=%lld acc=%d\n", i,
    // nodes[i].go[0], nodes[i].len[0], nodes[i].go[1], nodes[i].len[1],
    // nodes[i].ways, (int)nodes[i].acc);
  }
  assert(nodes[m + 5].ways >= bound);
  return {m + 5, 0};
}

int main() {
  i64 n, mod;
  int q;
  while (scanf("%lld%d%lld", &n, &q, &mod) == 3) {
    std::vector<i64> p(q);
    for (int i = 0; i < q; ++i)
      scanf("%lld", &p[i]);
    if (n <= 4)
      solve_small(n, mod, p);
    else {
      i64 bound = *std::max_element(p.begin(), p.end());
      int start;
      i64 nn;
      std::tie(start, nn) = build(n, mod, bound);
      // printf("start=%d nn=%lld\n", start, nn);
      i64 fn = fib(n, mod).second, prefix = 0;
      if (nn > 0) {
        prefix = fib(nn, mod).second;
        i64 x = nn / 2 - 1;
        if (x > 1) {
          prefix -= fib(x * 2 + 1, mod).second - 1;
        }
      }
      for (auto w : p) {
        i64 ret = fn;
        if (n % 2 == 1) {
          if (nn && w == 1)
            ret -= 1;
          else
            ret -= prefix;
          if (nn)
            w -= 1;
        } else {
          i64 cnt = std::max<i64>(0, nn / 2 - 2);
          if (w <= cnt) {
            i64 ww = w * 2 + 2, x = w;
            ret -= fib(ww, mod).second;
            if (x >= 1)
              ret += fib(x * 2 + 1, mod).second - 1;
            w = 0;
          } else {
            w -= cnt;
            ret -= prefix;
          }
        }
        for (int o = start; w;) {
          assert(w > 0 && o != -1);
          if (nodes[o].acc)
            --w;
          if (w == 0)
            break;
          int x = nodes[o].go[0], y = nodes[o].go[1];
          if (x != -1 && nodes[x].ways >= w) {
            ret -= nodes[o].len[0];
            o = x;
          } else {
            if (x != -1)
              w -= nodes[x].ways;
            ret -= nodes[o].len[1];
            o = y;
          }
        }
        ret %= mod;
        ret += mod + 1;
        ret %= mod;
        printf("%lld ", ret);
      }
      puts("");
    }
  }
  return 0;
}
