#include <algorithm>
#include <cassert>
#include <cstdio>
#include <functional>
#include <vector>

using pii = std::pair<int, int>;

const int N = 4e5 + 10, inf = 1e9;

struct Node {
  int mi, tag;
  void mark(int v) { mi = tag = v; }
} T[N << 2];

int ys[N], ny;

void build(int o, int l, int r) {
  T[o].mi = -inf;
  T[o].tag = inf;
  if (l + 1 == r)
    return;
  int m = (l + r) >> 1;
  build(o << 1, l, m);
  build(o << 1 | 1, m, r);
}

void push(int o) {
  if (T[o].tag != inf) {
    T[o << 1].mark(T[o].tag);
    T[o << 1 | 1].mark(T[o].tag);
    T[o].tag = inf;
  }
}

void ins(int o, int l, int r, int L, int R, int v) {
  if (L <= l && r <= R) {
    T[o].mark(v);
    return;
  }
  push(o);
  int m = (l + r) >> 1;
  if (L < m)
    ins(o << 1, l, m, L, R, v);
  if (R > m)
    ins(o << 1 | 1, m, r, L, R, v);
  T[o].mi = std::min(T[o << 1].mi, T[o << 1 | 1].mi);
}

std::vector<std::tuple<int, int, int>> segs;
void query(int o, int l, int r, int L, int R) {
  if (L <= l && r <= R) {
    segs.emplace_back(o, l, r);
    return;
  }
  push(o);
  int m = (l + r) >> 1;
  if (L < m)
    query(o << 1, l, m, L, R);
  if (R > m)
    query(o << 1 | 1, m, r, L, R);
}

void print(int o, int l, int r) {
  if (l + 1 == r) {
    printf("[%d, %d]=%d\n", ys[l], ys[r], T[o].mi);
    return;
  }
  push(o);
  int m = (l + r) >> 1;
  print(o << 1, l, m);
  print(o << 1 | 1, m, r);
}

int main() {
  int n, m;
  while (scanf("%d%d", &n, &m) == 2) {
    std::vector<pii> pts(n), qry(m);
    ny = 0;
    for (int i = 0; i < n; ++i) {
      scanf("%d%d", &pts[i].first, &pts[i].second);
      ys[ny++] = pts[i].second;
    }
    for (int i = 0; i < m; ++i) {
      scanf("%d%d", &qry[i].first, &qry[i].second);
      ys[ny++] = qry[i].second;
    }
    std::sort(ys, ys + ny);
    ny = std::unique(ys, ys + ny) - ys;
    ys[ny] = inf;
    // for (int i = 0; i < ny; ++i) printf("%d ", ys[i]);
    // puts("");

    std::vector<pii> ver, low(m);
    ver.reserve(n);
    for (int i = 0; i < m; ++i) {
      qry[i].second = std::lower_bound(ys, ys + ny, qry[i].second) - ys;
      low[i].first = qry[i].first;
      low[i].second = i;
    }
    for (int i = 0; i < n; ++i) {
      pts[i].second = std::lower_bound(ys, ys + ny, pts[i].second) - ys;
      if (pts[i].first == pts[(i + 1) % n].first) {
        ver.emplace_back(pts[i].first, i);
      }
    }
    std::sort(ver.begin(), ver.end());
    std::sort(low.begin(), low.end());
    std::vector<int> ret(m, -1);
    build(1, 0, ny);
    while (!low.empty()) {
      while (!ver.empty() && ver.back().first >= low.back().first) {
        int i = ver.back().second, j = (i + 1) % n;
        // printf("x=%d yi=%d yj=%d\n", pts[i].first, pts[i].second,
        // pts[j].second);
        if (pts[i].second < pts[j].second) { // enter
          ins(1, 0, ny, pts[i].second, pts[j].second, pts[i].first);
        } else { // leave
          ins(1, 0, ny, pts[j].second, pts[i].second, -inf);
        }
        ver.pop_back();
      }
      // print(1, 0, ny);
      int e = low.back().second;
      segs.clear();
      query(1, 0, ny, qry[e].second, ny);
      int xe = qry[e].first, ye = ys[qry[e].second];
      int px = inf;
      for (auto &s : segs) {
        int o, l, r;
        std::tie(o, l, r) = s;
        int yl = ys[l], yr = ys[r];
        int nx = std::min(px, T[o].mi);
        if ((yl - ye <= nx - xe && nx - xe <= yr - ye) ||
            (nx - xe <= yl - ye && yl - ye < px - xe)) {
          // printf("found yl=%d yr=%d mx=%d\n", ys[l], ys[r], T[o].mi);
          while (l + 1 != r) {
            push(o);
            int m = (l + r) >> 1, ym = ys[m];
            int mx = std::min(px, T[o << 1].mi);
            // printf("l=%d r=%d m=%d mx=%d ym=%d\n", l, r, m, mx, ym);
            if (mx - xe <= ym - ye) { // go left
              o = o << 1;
              r = m;
              yr = ym;
              nx = mx;
            } else { // go right
              o = o << 1 | 1;
              l = m;
              yl = ym;
              px = mx;
            }
          }
          ret[e] = std::max(yl - ye, nx - xe);
          break;
        }
        px = nx;
      }
      low.pop_back();
    }
    for (int i = 0; i < m; ++i) {
      printf("%d\n", ret[i]);
    }
  }
  return 0;
}
