#include <algorithm>
#include <cstdio>
#include <cstring>
#include <vector>

using i64 = long long;

const int N = 1e6 + 10;

i64 sum[N], ret[N];
int rep_x[N], rep_y[N];
int x[2][N], y[2][N];
int done[N], mark[N];
int a[N], n, m;

void prefix_sum(i64 *s, int n, int m) {
  for (int e = 0; e < n * m; ++e) {
    if (e >= m) {
      s[e] += s[e - m];
    }
    if (e % m) {
      s[e] += s[e - 1];
    }
    if (e >= m && e % m) {
      s[e] -= s[e - m - 1];
    }
  }
}

inline i64 rect_prefix_sum(i64 *s, int x, int y) {
  return (x < 0 || y < 0) ? 0 : s[x * m + y];
}

i64 rect_sum(i64 *s, int o) {
  return rect_prefix_sum(s, x[1][o], y[1][o]) -
         rect_prefix_sum(s, x[1][o], y[0][o] - 1) -
         rect_prefix_sum(s, x[0][o] - 1, y[1][o]) +
         rect_prefix_sum(s, x[0][o] - 1, y[0][o] - 1);
}

bool inside(int o, int u, int v) {
  return x[0][o] <= u && u <= x[1][o] && y[0][o] <= v && v <= y[1][o];
}

template <int ix, int iy> void prepare() {
  memset(sum, 0, sizeof(*sum) * n * m);
  for (int r = 0; r < n * m; ++r)
    if (mark[r]) {
      sum[x[ix][r] * m + y[iy][r]] += r + 1;
    }
  prefix_sum(sum, n, m);
  for (int i = 0; i < n * m; ++i)
    if (mark[i]) {
      ret[i] += rect_sum(sum, i);
    }
}

void init(int n, int m) {
  for (int i = 0; i < n * m; ++i) {
    x[0][i] = y[0][i] = n * m;
    x[1][i] = y[1][i] = -1;
    mark[i] = 0;
    ret[i] = 0;
  }
}

void handle_inside(int a, int b) {
  if (done[b])
    return;
  done[b] = true;
  int status[2][2], extra = 0, cnt_ba = 0, cnt_ab = 0;
  for (int ix = 0; ix < 2; ++ix) {
    for (int iy = 0; iy < 2; ++iy) {
      status[ix][iy] = inside(a, x[ix][b], y[iy][b]);
      cnt_ba += status[ix][iy];
      cnt_ab += inside(b, x[ix][a], y[iy][a]);
      extra += x[ix][a] == x[ix][b] && y[iy][a] == y[iy][b];
    }
  }
  // extra for inside
  if (inside(a, rep_x[b], rep_y[b]) && (!status[0][0] || !status[1][1])) {
    ret[a] += 4 * (b + 1);
  }
  // extra for corner
  if (status[0][0] && status[1][1]) {
    ret[b] -= extra * (a + 1);
    return;
  }
  // edge case
  // printf("a=%d b=%d cnt_ba=%d cnt_ab=%d\n", a, b, cnt_ba, cnt_ab);
  if (cnt_ba == 2) {
    ret[a] -= b + 1;
  }
  if (cnt_ba == 2 && cnt_ab == 0) {
    ret[b] += a + 1;
  }
}

int main() {
  while (scanf("%d%d", &n, &m) == 2) {
    init(n, m);
    memset(sum, 0, sizeof(*sum) * n * m);
    int cnt = 0;
    for (int i = 0, v; i < n; ++i) {
      for (int j = 0; j < m; ++j) {
        scanf("%d", &v);
        a[i * m + j] = --v;
        x[0][v] = std::min(x[0][v], i);
        x[1][v] = std::max(x[1][v], i);
        y[0][v] = std::min(y[0][v], j);
        y[1][v] = std::max(y[1][v], j);
        if (!mark[v]) {
          mark[v] = 1;
          rep_x[v] = i;
          rep_y[v] = j;
          sum[i * m + j] -= 4 * (v + 1);
          cnt += 1;
        }
      }
    }
    for (int i = 0; i < n * m; ++i)
      if (mark[i]) {
        for (int ix = 0; ix < 2; ++ix) {
          for (int iy = 0; iy < 2; ++iy) {
            sum[x[ix][i] * m + y[iy][i]] += i + 1;
          }
        }
      }

    // solve inside
    memset(done, 0, sizeof(*done) * n * m);
    prefix_sum(sum, n, m);
    for (int i = 0; i < n * m; ++i)
      if (mark[i]) {
        done[i] = true;
        ret[i] += rect_sum(sum, i);
        for (int xx = x[0][i]; xx <= x[1][i]; ++xx) {
          handle_inside(i, a[xx * m + y[0][i]]);
          handle_inside(i, a[xx * m + y[1][i]]);
        }
        for (int yy = y[0][i]; yy <= y[1][i]; ++yy) {
          handle_inside(i, a[x[0][i] * m + yy]);
          handle_inside(i, a[x[1][i] * m + yy]);
        }
        for (int xx = x[0][i]; xx <= x[1][i]; ++xx) {
          done[a[xx * m + y[0][i]]] = false;
          done[a[xx * m + y[1][i]]] = false;
        }
        for (int yy = y[0][i]; yy <= y[1][i]; ++yy) {
          done[a[x[0][i] * m + yy]] = false;
          done[a[x[1][i] * m + yy]] = false;
        }
        done[i] = false;
      }

    i64 sum = 0;
    cnt = 0;
    for (int i = 0; i < n * m; ++i)
      if (mark[i]) {
        sum += (i + 1) ^ ret[i];
        if (ret[i])
          ++cnt;
      }
    printf("%lld\n", sum);
  }
  return 0;
}
